# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, conlist
from .v1_label_selector import V1LabelSelector
from .v1_persistent_volume_claim import V1PersistentVolumeClaim
from .v1_pod_template_spec import V1PodTemplateSpec
from .v1_stateful_set_ordinals import V1StatefulSetOrdinals
from .v1_stateful_set_persistent_volume_claim_retention_policy import V1StatefulSetPersistentVolumeClaimRetentionPolicy
from .v1_stateful_set_update_strategy import V1StatefulSetUpdateStrategy

class V1StatefulSetSpec(BaseModel):
    """
    A StatefulSetSpec is the specification of a StatefulSet.  # noqa: E501
    """
    min_ready_seconds: Optional[StrictInt] = Field(None, alias="minReadySeconds", description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)")
    ordinals: Optional[V1StatefulSetOrdinals] = None
    persistent_volume_claim_retention_policy: Optional[V1StatefulSetPersistentVolumeClaimRetentionPolicy] = Field(None, alias="persistentVolumeClaimRetentionPolicy")
    pod_management_policy: Optional[StrictStr] = Field(None, alias="podManagementPolicy", description="podManagementPolicy controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down. The default policy is `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before continuing. When scaling down, the pods are removed in the opposite order. The alternative policy is `Parallel` which will create pods in parallel to match the desired scale without waiting, and on scale down will delete all pods at once.")
    replicas: Optional[StrictInt] = Field(None, description="replicas is the desired number of replicas of the given Template. These are replicas in the sense that they are instantiations of the same Template, but individual replicas also have a consistent identity. If unspecified, defaults to 1.")
    revision_history_limit: Optional[StrictInt] = Field(None, alias="revisionHistoryLimit", description="revisionHistoryLimit is the maximum number of revisions that will be maintained in the StatefulSet's revision history. The revision history consists of all revisions not represented by a currently applied StatefulSetSpec version. The default value is 10.")
    selector: V1LabelSelector = Field(...)
    service_name: StrictStr = Field(..., alias="serviceName", description="serviceName is the name of the service that governs this StatefulSet. This service must exist before the StatefulSet, and is responsible for the network identity of the set. Pods get DNS/hostnames that follow the pattern: pod-specific-string.serviceName.default.svc.cluster.local where \"pod-specific-string\" is managed by the StatefulSet controller.")
    template: V1PodTemplateSpec = Field(...)
    update_strategy: Optional[V1StatefulSetUpdateStrategy] = Field(None, alias="updateStrategy")
    volume_claim_templates: Optional[conlist(V1PersistentVolumeClaim)] = Field(None, alias="volumeClaimTemplates", description="volumeClaimTemplates is a list of claims that pods are allowed to reference. The StatefulSet controller is responsible for mapping network identities to claims in a way that maintains the identity of a pod. Every claim in this list must have at least one matching (by name) volumeMount in one container in the template. A claim in this list takes precedence over any volumes in the template, with the same name.")
    __properties = ["minReadySeconds", "ordinals", "persistentVolumeClaimRetentionPolicy", "podManagementPolicy", "replicas", "revisionHistoryLimit", "selector", "serviceName", "template", "updateStrategy", "volumeClaimTemplates"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1StatefulSetSpec:
        """Create an instance of V1StatefulSetSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of ordinals
        if self.ordinals:
            _dict['ordinals'] = self.ordinals.to_dict()
        # override the default output from pydantic by calling `to_dict()` of persistent_volume_claim_retention_policy
        if self.persistent_volume_claim_retention_policy:
            _dict['persistentVolumeClaimRetentionPolicy'] = self.persistent_volume_claim_retention_policy.to_dict()
        # override the default output from pydantic by calling `to_dict()` of selector
        if self.selector:
            _dict['selector'] = self.selector.to_dict()
        # override the default output from pydantic by calling `to_dict()` of template
        if self.template:
            _dict['template'] = self.template.to_dict()
        # override the default output from pydantic by calling `to_dict()` of update_strategy
        if self.update_strategy:
            _dict['updateStrategy'] = self.update_strategy.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in volume_claim_templates (list)
        _items = []
        if self.volume_claim_templates:
            for _item in self.volume_claim_templates:
                if _item:
                    _items.append(_item.to_dict())
            _dict['volumeClaimTemplates'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1StatefulSetSpec:
        """Create an instance of V1StatefulSetSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1StatefulSetSpec.parse_obj(obj)

        _obj = V1StatefulSetSpec.parse_obj({
            "min_ready_seconds": obj.get("minReadySeconds"),
            "ordinals": V1StatefulSetOrdinals.from_dict(obj.get("ordinals")) if obj.get("ordinals") is not None else None,
            "persistent_volume_claim_retention_policy": V1StatefulSetPersistentVolumeClaimRetentionPolicy.from_dict(obj.get("persistentVolumeClaimRetentionPolicy")) if obj.get("persistentVolumeClaimRetentionPolicy") is not None else None,
            "pod_management_policy": obj.get("podManagementPolicy"),
            "replicas": obj.get("replicas"),
            "revision_history_limit": obj.get("revisionHistoryLimit"),
            "selector": V1LabelSelector.from_dict(obj.get("selector")) if obj.get("selector") is not None else None,
            "service_name": obj.get("serviceName"),
            "template": V1PodTemplateSpec.from_dict(obj.get("template")) if obj.get("template") is not None else None,
            "update_strategy": V1StatefulSetUpdateStrategy.from_dict(obj.get("updateStrategy")) if obj.get("updateStrategy") is not None else None,
            "volume_claim_templates": [V1PersistentVolumeClaim.from_dict(_item) for _item in obj.get("volumeClaimTemplates")] if obj.get("volumeClaimTemplates") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_label_selector import V1LabelSelector

class V1PodAffinityTerm(BaseModel):
    """
    Defines a set of pods (namely those matching the labelSelector relative to the given namespace(s)) that this pod should be co-located (affinity) or not co-located (anti-affinity) with, where co-located is defined as running on a node whose value of the label with key <topologyKey> matches that of any node on which a pod of the set of pods is running  # noqa: E501
    """
    label_selector: Optional[V1LabelSelector] = Field(None, alias="labelSelector")
    match_label_keys: Optional[conlist(StrictStr)] = Field(None, alias="matchLabelKeys", description="MatchLabelKeys is a set of pod label keys to select which pods will be taken into consideration. The keys are used to lookup values from the incoming pod labels, those key-value labels are merged with `LabelSelector` as `key in (value)` to select the group of existing pods which pods will be taken into consideration for the incoming pod's pod (anti) affinity. Keys that don't exist in the incoming pod labels will be ignored. The default value is empty. The same key is forbidden to exist in both MatchLabelKeys and LabelSelector. Also, MatchLabelKeys cannot be set when LabelSelector isn't set. This is an alpha field and requires enabling MatchLabelKeysInPodAffinity feature gate.")
    mismatch_label_keys: Optional[conlist(StrictStr)] = Field(None, alias="mismatchLabelKeys", description="MismatchLabelKeys is a set of pod label keys to select which pods will be taken into consideration. The keys are used to lookup values from the incoming pod labels, those key-value labels are merged with `LabelSelector` as `key notin (value)` to select the group of existing pods which pods will be taken into consideration for the incoming pod's pod (anti) affinity. Keys that don't exist in the incoming pod labels will be ignored. The default value is empty. The same key is forbidden to exist in both MismatchLabelKeys and LabelSelector. Also, MismatchLabelKeys cannot be set when LabelSelector isn't set. This is an alpha field and requires enabling MatchLabelKeysInPodAffinity feature gate.")
    namespace_selector: Optional[V1LabelSelector] = Field(None, alias="namespaceSelector")
    namespaces: Optional[conlist(StrictStr)] = Field(None, description="namespaces specifies a static list of namespace names that the term applies to. The term is applied to the union of the namespaces listed in this field and the ones selected by namespaceSelector. null or empty namespaces list and null namespaceSelector means \"this pod's namespace\".")
    topology_key: StrictStr = Field(..., alias="topologyKey", description="This pod should be co-located (affinity) or not co-located (anti-affinity) with the pods matching the labelSelector in the specified namespaces, where co-located is defined as running on a node whose value of the label with key topologyKey matches that of any node on which any of the selected pods is running. Empty topologyKey is not allowed.")
    __properties = ["labelSelector", "matchLabelKeys", "mismatchLabelKeys", "namespaceSelector", "namespaces", "topologyKey"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PodAffinityTerm:
        """Create an instance of V1PodAffinityTerm from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of label_selector
        if self.label_selector:
            _dict['labelSelector'] = self.label_selector.to_dict()
        # override the default output from pydantic by calling `to_dict()` of namespace_selector
        if self.namespace_selector:
            _dict['namespaceSelector'] = self.namespace_selector.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PodAffinityTerm:
        """Create an instance of V1PodAffinityTerm from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PodAffinityTerm.parse_obj(obj)

        _obj = V1PodAffinityTerm.parse_obj({
            "label_selector": V1LabelSelector.from_dict(obj.get("labelSelector")) if obj.get("labelSelector") is not None else None,
            "match_label_keys": obj.get("matchLabelKeys"),
            "mismatch_label_keys": obj.get("mismatchLabelKeys"),
            "namespace_selector": V1LabelSelector.from_dict(obj.get("namespaceSelector")) if obj.get("namespaceSelector") is not None else None,
            "namespaces": obj.get("namespaces"),
            "topology_key": obj.get("topologyKey")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_container_status import V1ContainerStatus
from .v1_host_ip import V1HostIP
from .v1_pod_condition import V1PodCondition
from .v1_pod_ip import V1PodIP
from .v1_pod_resource_claim_status import V1PodResourceClaimStatus

class V1PodStatus(BaseModel):
    """
    PodStatus represents information about the status of a pod. Status may trail the actual state of a system, especially if the node that hosts the pod cannot contact the control plane.  # noqa: E501
    """
    conditions: Optional[conlist(V1PodCondition)] = Field(None, description="Current service state of pod. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-conditions")
    container_statuses: Optional[conlist(V1ContainerStatus)] = Field(None, alias="containerStatuses", description="The list has one entry per container in the manifest. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-and-container-status")
    ephemeral_container_statuses: Optional[conlist(V1ContainerStatus)] = Field(None, alias="ephemeralContainerStatuses", description="Status for any ephemeral containers that have run in this pod.")
    host_ip: Optional[StrictStr] = Field(None, alias="hostIP", description="hostIP holds the IP address of the host to which the pod is assigned. Empty if the pod has not started yet. A pod can be assigned to a node that has a problem in kubelet which in turns mean that HostIP will not be updated even if there is a node is assigned to pod")
    host_ips: Optional[conlist(V1HostIP)] = Field(None, alias="hostIPs", description="hostIPs holds the IP addresses allocated to the host. If this field is specified, the first entry must match the hostIP field. This list is empty if the pod has not started yet. A pod can be assigned to a node that has a problem in kubelet which in turns means that HostIPs will not be updated even if there is a node is assigned to this pod.")
    init_container_statuses: Optional[conlist(V1ContainerStatus)] = Field(None, alias="initContainerStatuses", description="The list has one entry per init container in the manifest. The most recent successful init container will have ready = true, the most recently started container will have startTime set. More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-and-container-status")
    message: Optional[StrictStr] = Field(None, description="A human readable message indicating details about why the pod is in this condition.")
    nominated_node_name: Optional[StrictStr] = Field(None, alias="nominatedNodeName", description="nominatedNodeName is set only when this pod preempts other pods on the node, but it cannot be scheduled right away as preemption victims receive their graceful termination periods. This field does not guarantee that the pod will be scheduled on this node. Scheduler may decide to place the pod elsewhere if other nodes become available sooner. Scheduler may also decide to give the resources on this node to a higher priority pod that is created after preemption. As a result, this field may be different than PodSpec.nodeName when the pod is scheduled.")
    phase: Optional[StrictStr] = Field(None, description="The phase of a Pod is a simple, high-level summary of where the Pod is in its lifecycle. The conditions array, the reason and message fields, and the individual container status arrays contain more detail about the pod's status. There are five possible phase values:  Pending: The pod has been accepted by the Kubernetes system, but one or more of the container images has not been created. This includes time before being scheduled as well as time spent downloading images over the network, which could take a while. Running: The pod has been bound to a node, and all of the containers have been created. At least one container is still running, or is in the process of starting or restarting. Succeeded: All containers in the pod have terminated in success, and will not be restarted. Failed: All containers in the pod have terminated, and at least one container has terminated in failure. The container either exited with non-zero status or was terminated by the system. Unknown: For some reason the state of the pod could not be obtained, typically due to an error in communicating with the host of the pod.  More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle#pod-phase")
    pod_ip: Optional[StrictStr] = Field(None, alias="podIP", description="podIP address allocated to the pod. Routable at least within the cluster. Empty if not yet allocated.")
    pod_ips: Optional[conlist(V1PodIP)] = Field(None, alias="podIPs", description="podIPs holds the IP addresses allocated to the pod. If this field is specified, the 0th entry must match the podIP field. Pods may be allocated at most 1 value for each of IPv4 and IPv6. This list is empty if no IPs have been allocated yet.")
    qos_class: Optional[StrictStr] = Field(None, alias="qosClass", description="The Quality of Service (QOS) classification assigned to the pod based on resource requirements See PodQOSClass type for available QOS classes More info: https://kubernetes.io/docs/concepts/workloads/pods/pod-qos/#quality-of-service-classes")
    reason: Optional[StrictStr] = Field(None, description="A brief CamelCase message indicating details about why the pod is in this state. e.g. 'Evicted'")
    resize: Optional[StrictStr] = Field(None, description="Status of resources resize desired for pod's containers. It is empty if no resources resize is pending. Any changes to container resources will automatically set this to \"Proposed\"")
    resource_claim_statuses: Optional[conlist(V1PodResourceClaimStatus)] = Field(None, alias="resourceClaimStatuses", description="Status of resource claims.")
    start_time: Optional[datetime] = Field(None, alias="startTime", description="RFC 3339 date and time at which the object was acknowledged by the Kubelet. This is before the Kubelet pulled the container image(s) for the pod.")
    __properties = ["conditions", "containerStatuses", "ephemeralContainerStatuses", "hostIP", "hostIPs", "initContainerStatuses", "message", "nominatedNodeName", "phase", "podIP", "podIPs", "qosClass", "reason", "resize", "resourceClaimStatuses", "startTime"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PodStatus:
        """Create an instance of V1PodStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in conditions (list)
        _items = []
        if self.conditions:
            for _item in self.conditions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['conditions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in container_statuses (list)
        _items = []
        if self.container_statuses:
            for _item in self.container_statuses:
                if _item:
                    _items.append(_item.to_dict())
            _dict['containerStatuses'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in ephemeral_container_statuses (list)
        _items = []
        if self.ephemeral_container_statuses:
            for _item in self.ephemeral_container_statuses:
                if _item:
                    _items.append(_item.to_dict())
            _dict['ephemeralContainerStatuses'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in host_ips (list)
        _items = []
        if self.host_ips:
            for _item in self.host_ips:
                if _item:
                    _items.append(_item.to_dict())
            _dict['hostIPs'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in init_container_statuses (list)
        _items = []
        if self.init_container_statuses:
            for _item in self.init_container_statuses:
                if _item:
                    _items.append(_item.to_dict())
            _dict['initContainerStatuses'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in pod_ips (list)
        _items = []
        if self.pod_ips:
            for _item in self.pod_ips:
                if _item:
                    _items.append(_item.to_dict())
            _dict['podIPs'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in resource_claim_statuses (list)
        _items = []
        if self.resource_claim_statuses:
            for _item in self.resource_claim_statuses:
                if _item:
                    _items.append(_item.to_dict())
            _dict['resourceClaimStatuses'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PodStatus:
        """Create an instance of V1PodStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PodStatus.parse_obj(obj)

        _obj = V1PodStatus.parse_obj({
            "conditions": [V1PodCondition.from_dict(_item) for _item in obj.get("conditions")] if obj.get("conditions") is not None else None,
            "container_statuses": [V1ContainerStatus.from_dict(_item) for _item in obj.get("containerStatuses")] if obj.get("containerStatuses") is not None else None,
            "ephemeral_container_statuses": [V1ContainerStatus.from_dict(_item) for _item in obj.get("ephemeralContainerStatuses")] if obj.get("ephemeralContainerStatuses") is not None else None,
            "host_ip": obj.get("hostIP"),
            "host_ips": [V1HostIP.from_dict(_item) for _item in obj.get("hostIPs")] if obj.get("hostIPs") is not None else None,
            "init_container_statuses": [V1ContainerStatus.from_dict(_item) for _item in obj.get("initContainerStatuses")] if obj.get("initContainerStatuses") is not None else None,
            "message": obj.get("message"),
            "nominated_node_name": obj.get("nominatedNodeName"),
            "phase": obj.get("phase"),
            "pod_ip": obj.get("podIP"),
            "pod_ips": [V1PodIP.from_dict(_item) for _item in obj.get("podIPs")] if obj.get("podIPs") is not None else None,
            "qos_class": obj.get("qosClass"),
            "reason": obj.get("reason"),
            "resize": obj.get("resize"),
            "resource_claim_statuses": [V1PodResourceClaimStatus.from_dict(_item) for _item in obj.get("resourceClaimStatuses")] if obj.get("resourceClaimStatuses") is not None else None,
            "start_time": obj.get("startTime")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field
from .v1_node_affinity import V1NodeAffinity
from .v1_pod_affinity import V1PodAffinity
from .v1_pod_anti_affinity import V1PodAntiAffinity

class V1Affinity(BaseModel):
    """
    Affinity is a group of affinity scheduling rules.  # noqa: E501
    """
    node_affinity: Optional[V1NodeAffinity] = Field(None, alias="nodeAffinity")
    pod_affinity: Optional[V1PodAffinity] = Field(None, alias="podAffinity")
    pod_anti_affinity: Optional[V1PodAntiAffinity] = Field(None, alias="podAntiAffinity")
    __properties = ["nodeAffinity", "podAffinity", "podAntiAffinity"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1Affinity:
        """Create an instance of V1Affinity from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of node_affinity
        if self.node_affinity:
            _dict['nodeAffinity'] = self.node_affinity.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pod_affinity
        if self.pod_affinity:
            _dict['podAffinity'] = self.pod_affinity.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pod_anti_affinity
        if self.pod_anti_affinity:
            _dict['podAntiAffinity'] = self.pod_anti_affinity.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1Affinity:
        """Create an instance of V1Affinity from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1Affinity.parse_obj(obj)

        _obj = V1Affinity.parse_obj({
            "node_affinity": V1NodeAffinity.from_dict(obj.get("nodeAffinity")) if obj.get("nodeAffinity") is not None else None,
            "pod_affinity": V1PodAffinity.from_dict(obj.get("podAffinity")) if obj.get("podAffinity") is not None else None,
            "pod_anti_affinity": V1PodAntiAffinity.from_dict(obj.get("podAntiAffinity")) if obj.get("podAntiAffinity") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic import BaseModel, Field, StrictStr

class V1MatchCondition(BaseModel):
    """
    MatchCondition represents a condition which must by fulfilled for a request to be sent to a webhook.  # noqa: E501
    """
    expression: StrictStr = Field(..., description="Expression represents the expression which will be evaluated by CEL. Must evaluate to bool. CEL expressions have access to the contents of the AdmissionRequest and Authorizer, organized into CEL variables:  'object' - The object from the incoming request. The value is null for DELETE requests. 'oldObject' - The existing object. The value is null for CREATE requests. 'request' - Attributes of the admission request(/pkg/apis/admission/types.go#AdmissionRequest). 'authorizer' - A CEL Authorizer. May be used to perform authorization checks for the principal (user or service account) of the request.   See https://pkg.go.dev/k8s.io/apiserver/pkg/cel/library#Authz 'authorizer.requestResource' - A CEL ResourceCheck constructed from the 'authorizer' and configured with the   request resource. Documentation on CEL: https://kubernetes.io/docs/reference/using-api/cel/  Required.")
    name: StrictStr = Field(..., description="Name is an identifier for this match condition, used for strategic merging of MatchConditions, as well as providing an identifier for logging purposes. A good name should be descriptive of the associated expression. Name must be a qualified name consisting of alphanumeric characters, '-', '_' or '.', and must start and end with an alphanumeric character (e.g. 'MyName',  or 'my.name',  or '123-abc', regex used for validation is '([A-Za-z0-9][-A-Za-z0-9_.]*)?[A-Za-z0-9]') with an optional DNS subdomain prefix and '/' (e.g. 'example.com/MyName')  Required.")
    __properties = ["expression", "name"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1MatchCondition:
        """Create an instance of V1MatchCondition from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1MatchCondition:
        """Create an instance of V1MatchCondition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1MatchCondition.parse_obj(obj)

        _obj = V1MatchCondition.parse_obj({
            "expression": obj.get("expression"),
            "name": obj.get("name")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictStr, conbytes, constr, validator
from .v1_object_meta import V1ObjectMeta

class V1Secret(BaseModel):
    """
    Secret holds secret data of a certain type. The total bytes of the values in the Data field must be less than MaxSecretSize bytes.  # noqa: E501
    """
    api_version: Optional[StrictStr] = Field(None, alias="apiVersion", description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources")
    data: Optional[Dict[str, Union[conbytes(strict=True), constr(strict=True)]]] = Field(None, description="Data contains the secret data. Each key must consist of alphanumeric characters, '-', '_' or '.'. The serialized form of the secret data is a base64 encoded string, representing the arbitrary (possibly non-string) data value here. Described in https://tools.ietf.org/html/rfc4648#section-4")
    immutable: Optional[StrictBool] = Field(None, description="Immutable, if set to true, ensures that data stored in the Secret cannot be updated (only object metadata can be modified). If not set to true, the field can be modified at any time. Defaulted to nil.")
    kind: Optional[StrictStr] = Field(None, description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds")
    metadata: Optional[V1ObjectMeta] = None
    string_data: Optional[Dict[str, StrictStr]] = Field(None, alias="stringData", description="stringData allows specifying non-binary secret data in string form. It is provided as a write-only input field for convenience. All keys and values are merged into the data field on write, overwriting any existing values. The stringData field is never output when reading from the API.")
    type: Optional[StrictStr] = Field(None, description="Used to facilitate programmatic handling of secret data. More info: https://kubernetes.io/docs/concepts/configuration/secret/#secret-types")
    __properties = ["apiVersion", "data", "immutable", "kind", "metadata", "stringData", "type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1Secret:
        """Create an instance of V1Secret from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1Secret:
        """Create an instance of V1Secret from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1Secret.parse_obj(obj)

        _obj = V1Secret.parse_obj({
            "api_version": obj.get("apiVersion"),
            "data": obj.get("data"),
            "immutable": obj.get("immutable"),
            "kind": obj.get("kind"),
            "metadata": V1ObjectMeta.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None,
            "string_data": obj.get("stringData"),
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, conlist
from .v1beta3_non_resource_policy_rule import V1beta3NonResourcePolicyRule
from .v1beta3_resource_policy_rule import V1beta3ResourcePolicyRule
from .v1beta3_subject import V1beta3Subject

class V1beta3PolicyRulesWithSubjects(BaseModel):
    """
    PolicyRulesWithSubjects prescribes a test that applies to a request to an apiserver. The test considers the subject making the request, the verb being requested, and the resource to be acted upon. This PolicyRulesWithSubjects matches a request if and only if both (a) at least one member of subjects matches the request and (b) at least one member of resourceRules or nonResourceRules matches the request.  # noqa: E501
    """
    non_resource_rules: Optional[conlist(V1beta3NonResourcePolicyRule)] = Field(None, alias="nonResourceRules", description="`nonResourceRules` is a list of NonResourcePolicyRules that identify matching requests according to their verb and the target non-resource URL.")
    resource_rules: Optional[conlist(V1beta3ResourcePolicyRule)] = Field(None, alias="resourceRules", description="`resourceRules` is a slice of ResourcePolicyRules that identify matching requests according to their verb and the target resource. At least one of `resourceRules` and `nonResourceRules` has to be non-empty.")
    subjects: conlist(V1beta3Subject) = Field(..., description="subjects is the list of normal user, serviceaccount, or group that this rule cares about. There must be at least one member in this slice. A slice that includes both the system:authenticated and system:unauthenticated user groups matches every request. Required.")
    __properties = ["nonResourceRules", "resourceRules", "subjects"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1beta3PolicyRulesWithSubjects:
        """Create an instance of V1beta3PolicyRulesWithSubjects from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in non_resource_rules (list)
        _items = []
        if self.non_resource_rules:
            for _item in self.non_resource_rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['nonResourceRules'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in resource_rules (list)
        _items = []
        if self.resource_rules:
            for _item in self.resource_rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['resourceRules'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in subjects (list)
        _items = []
        if self.subjects:
            for _item in self.subjects:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subjects'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1beta3PolicyRulesWithSubjects:
        """Create an instance of V1beta3PolicyRulesWithSubjects from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1beta3PolicyRulesWithSubjects.parse_obj(obj)

        _obj = V1beta3PolicyRulesWithSubjects.parse_obj({
            "non_resource_rules": [V1beta3NonResourcePolicyRule.from_dict(_item) for _item in obj.get("nonResourceRules")] if obj.get("nonResourceRules") is not None else None,
            "resource_rules": [V1beta3ResourcePolicyRule.from_dict(_item) for _item in obj.get("resourceRules")] if obj.get("resourceRules") is not None else None,
            "subjects": [V1beta3Subject.from_dict(_item) for _item in obj.get("subjects")] if obj.get("subjects") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_endpoint_conditions import V1EndpointConditions
from .v1_endpoint_hints import V1EndpointHints
from .v1_object_reference import V1ObjectReference

class V1Endpoint(BaseModel):
    """
    Endpoint represents a single logical \"backend\" implementing a service.  # noqa: E501
    """
    addresses: conlist(StrictStr) = Field(..., description="addresses of this endpoint. The contents of this field are interpreted according to the corresponding EndpointSlice addressType field. Consumers must handle different types of addresses in the context of their own capabilities. This must contain at least one address but no more than 100. These are all assumed to be fungible and clients may choose to only use the first element. Refer to: https://issue.k8s.io/106267")
    conditions: Optional[V1EndpointConditions] = None
    deprecated_topology: Optional[Dict[str, StrictStr]] = Field(None, alias="deprecatedTopology", description="deprecatedTopology contains topology information part of the v1beta1 API. This field is deprecated, and will be removed when the v1beta1 API is removed (no sooner than kubernetes v1.24).  While this field can hold values, it is not writable through the v1 API, and any attempts to write to it will be silently ignored. Topology information can be found in the zone and nodeName fields instead.")
    hints: Optional[V1EndpointHints] = None
    hostname: Optional[StrictStr] = Field(None, description="hostname of this endpoint. This field may be used by consumers of endpoints to distinguish endpoints from each other (e.g. in DNS names). Multiple endpoints which use the same hostname should be considered fungible (e.g. multiple A values in DNS). Must be lowercase and pass DNS Label (RFC 1123) validation.")
    node_name: Optional[StrictStr] = Field(None, alias="nodeName", description="nodeName represents the name of the Node hosting this endpoint. This can be used to determine endpoints local to a Node.")
    target_ref: Optional[V1ObjectReference] = Field(None, alias="targetRef")
    zone: Optional[StrictStr] = Field(None, description="zone is the name of the Zone this endpoint exists in.")
    __properties = ["addresses", "conditions", "deprecatedTopology", "hints", "hostname", "nodeName", "targetRef", "zone"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1Endpoint:
        """Create an instance of V1Endpoint from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of conditions
        if self.conditions:
            _dict['conditions'] = self.conditions.to_dict()
        # override the default output from pydantic by calling `to_dict()` of hints
        if self.hints:
            _dict['hints'] = self.hints.to_dict()
        # override the default output from pydantic by calling `to_dict()` of target_ref
        if self.target_ref:
            _dict['targetRef'] = self.target_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1Endpoint:
        """Create an instance of V1Endpoint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1Endpoint.parse_obj(obj)

        _obj = V1Endpoint.parse_obj({
            "addresses": obj.get("addresses"),
            "conditions": V1EndpointConditions.from_dict(obj.get("conditions")) if obj.get("conditions") is not None else None,
            "deprecated_topology": obj.get("deprecatedTopology"),
            "hints": V1EndpointHints.from_dict(obj.get("hints")) if obj.get("hints") is not None else None,
            "hostname": obj.get("hostname"),
            "node_name": obj.get("nodeName"),
            "target_ref": V1ObjectReference.from_dict(obj.get("targetRef")) if obj.get("targetRef") is not None else None,
            "zone": obj.get("zone")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr

class V1KeyToPath(BaseModel):
    """
    Maps a string key to a path within a volume.  # noqa: E501
    """
    key: StrictStr = Field(..., description="key is the key to project.")
    mode: Optional[StrictInt] = Field(None, description="mode is Optional: mode bits used to set permissions on this file. Must be an octal value between 0000 and 0777 or a decimal value between 0 and 511. YAML accepts both octal and decimal values, JSON requires decimal values for mode bits. If not specified, the volume defaultMode will be used. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.")
    path: StrictStr = Field(..., description="path is the relative path of the file to map the key to. May not be an absolute path. May not contain the path element '..'. May not start with the string '..'.")
    __properties = ["key", "mode", "path"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1KeyToPath:
        """Create an instance of V1KeyToPath from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1KeyToPath:
        """Create an instance of V1KeyToPath from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1KeyToPath.parse_obj(obj)

        _obj = V1KeyToPath.parse_obj({
            "key": obj.get("key"),
            "mode": obj.get("mode"),
            "path": obj.get("path")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_toleration import V1Toleration

class V1Scheduling(BaseModel):
    """
    Scheduling specifies the scheduling constraints for nodes supporting a RuntimeClass.  # noqa: E501
    """
    node_selector: Optional[Dict[str, StrictStr]] = Field(None, alias="nodeSelector", description="nodeSelector lists labels that must be present on nodes that support this RuntimeClass. Pods using this RuntimeClass can only be scheduled to a node matched by this selector. The RuntimeClass nodeSelector is merged with a pod's existing nodeSelector. Any conflicts will cause the pod to be rejected in admission.")
    tolerations: Optional[conlist(V1Toleration)] = Field(None, description="tolerations are appended (excluding duplicates) to pods running with this RuntimeClass during admission, effectively unioning the set of nodes tolerated by the pod and the RuntimeClass.")
    __properties = ["nodeSelector", "tolerations"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1Scheduling:
        """Create an instance of V1Scheduling from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in tolerations (list)
        _items = []
        if self.tolerations:
            for _item in self.tolerations:
                if _item:
                    _items.append(_item.to_dict())
            _dict['tolerations'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1Scheduling:
        """Create an instance of V1Scheduling from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1Scheduling.parse_obj(obj)

        _obj = V1Scheduling.parse_obj({
            "node_selector": obj.get("nodeSelector"),
            "tolerations": [V1Toleration.from_dict(_item) for _item in obj.get("tolerations")] if obj.get("tolerations") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr
from .v1_label_selector import V1LabelSelector
from .v1_pod_failure_policy import V1PodFailurePolicy
from .v1_pod_template_spec import V1PodTemplateSpec

class V1JobSpec(BaseModel):
    """
    JobSpec describes how the job execution will look like.  # noqa: E501
    """
    active_deadline_seconds: Optional[StrictInt] = Field(None, alias="activeDeadlineSeconds", description="Specifies the duration in seconds relative to the startTime that the job may be continuously active before the system tries to terminate it; value must be positive integer. If a Job is suspended (at creation or through an update), this timer will effectively be stopped and reset when the Job is resumed again.")
    backoff_limit: Optional[StrictInt] = Field(None, alias="backoffLimit", description="Specifies the number of retries before marking this job failed. Defaults to 6")
    backoff_limit_per_index: Optional[StrictInt] = Field(None, alias="backoffLimitPerIndex", description="Specifies the limit for the number of retries within an index before marking this index as failed. When enabled the number of failures per index is kept in the pod's batch.kubernetes.io/job-index-failure-count annotation. It can only be set when Job's completionMode=Indexed, and the Pod's restart policy is Never. The field is immutable. This field is beta-level. It can be used when the `JobBackoffLimitPerIndex` feature gate is enabled (enabled by default).")
    completion_mode: Optional[StrictStr] = Field(None, alias="completionMode", description="completionMode specifies how Pod completions are tracked. It can be `NonIndexed` (default) or `Indexed`.  `NonIndexed` means that the Job is considered complete when there have been .spec.completions successfully completed Pods. Each Pod completion is homologous to each other.  `Indexed` means that the Pods of a Job get an associated completion index from 0 to (.spec.completions - 1), available in the annotation batch.kubernetes.io/job-completion-index. The Job is considered complete when there is one successfully completed Pod for each index. When value is `Indexed`, .spec.completions must be specified and `.spec.parallelism` must be less than or equal to 10^5. In addition, The Pod name takes the form `$(job-name)-$(index)-$(random-string)`, the Pod hostname takes the form `$(job-name)-$(index)`.  More completion modes can be added in the future. If the Job controller observes a mode that it doesn't recognize, which is possible during upgrades due to version skew, the controller skips updates for the Job.")
    completions: Optional[StrictInt] = Field(None, description="Specifies the desired number of successfully finished pods the job should be run with.  Setting to null means that the success of any pod signals the success of all pods, and allows parallelism to have any positive value.  Setting to 1 means that parallelism is limited to 1 and the success of that pod signals the success of the job. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/")
    manual_selector: Optional[StrictBool] = Field(None, alias="manualSelector", description="manualSelector controls generation of pod labels and pod selectors. Leave `manualSelector` unset unless you are certain what you are doing. When false or unset, the system pick labels unique to this job and appends those labels to the pod template.  When true, the user is responsible for picking unique labels and specifying the selector.  Failure to pick a unique label may cause this and other jobs to not function correctly.  However, You may see `manualSelector=true` in jobs that were created with the old `extensions/v1beta1` API. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/#specifying-your-own-pod-selector")
    max_failed_indexes: Optional[StrictInt] = Field(None, alias="maxFailedIndexes", description="Specifies the maximal number of failed indexes before marking the Job as failed, when backoffLimitPerIndex is set. Once the number of failed indexes exceeds this number the entire Job is marked as Failed and its execution is terminated. When left as null the job continues execution of all of its indexes and is marked with the `Complete` Job condition. It can only be specified when backoffLimitPerIndex is set. It can be null or up to completions. It is required and must be less than or equal to 10^4 when is completions greater than 10^5. This field is beta-level. It can be used when the `JobBackoffLimitPerIndex` feature gate is enabled (enabled by default).")
    parallelism: Optional[StrictInt] = Field(None, description="Specifies the maximum desired number of pods the job should run at any given time. The actual number of pods running in steady state will be less than this number when ((.spec.completions - .status.successful) < .spec.parallelism), i.e. when the work left to do is less than max parallelism. More info: https://kubernetes.io/docs/concepts/workloads/controllers/jobs-run-to-completion/")
    pod_failure_policy: Optional[V1PodFailurePolicy] = Field(None, alias="podFailurePolicy")
    pod_replacement_policy: Optional[StrictStr] = Field(None, alias="podReplacementPolicy", description="podReplacementPolicy specifies when to create replacement Pods. Possible values are: - TerminatingOrFailed means that we recreate pods   when they are terminating (has a metadata.deletionTimestamp) or failed. - Failed means to wait until a previously created Pod is fully terminated (has phase   Failed or Succeeded) before creating a replacement Pod.  When using podFailurePolicy, Failed is the the only allowed value. TerminatingOrFailed and Failed are allowed values when podFailurePolicy is not in use. This is an beta field. To use this, enable the JobPodReplacementPolicy feature toggle. This is on by default.")
    selector: Optional[V1LabelSelector] = None
    suspend: Optional[StrictBool] = Field(None, description="suspend specifies whether the Job controller should create Pods or not. If a Job is created with suspend set to true, no Pods are created by the Job controller. If a Job is suspended after creation (i.e. the flag goes from false to true), the Job controller will delete all active Pods associated with this Job. Users must design their workload to gracefully handle this. Suspending a Job will reset the StartTime field of the Job, effectively resetting the ActiveDeadlineSeconds timer too. Defaults to false.")
    template: V1PodTemplateSpec = Field(...)
    ttl_seconds_after_finished: Optional[StrictInt] = Field(None, alias="ttlSecondsAfterFinished", description="ttlSecondsAfterFinished limits the lifetime of a Job that has finished execution (either Complete or Failed). If this field is set, ttlSecondsAfterFinished after the Job finishes, it is eligible to be automatically deleted. When the Job is being deleted, its lifecycle guarantees (e.g. finalizers) will be honored. If this field is unset, the Job won't be automatically deleted. If this field is set to zero, the Job becomes eligible to be deleted immediately after it finishes.")
    __properties = ["activeDeadlineSeconds", "backoffLimit", "backoffLimitPerIndex", "completionMode", "completions", "manualSelector", "maxFailedIndexes", "parallelism", "podFailurePolicy", "podReplacementPolicy", "selector", "suspend", "template", "ttlSecondsAfterFinished"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1JobSpec:
        """Create an instance of V1JobSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of pod_failure_policy
        if self.pod_failure_policy:
            _dict['podFailurePolicy'] = self.pod_failure_policy.to_dict()
        # override the default output from pydantic by calling `to_dict()` of selector
        if self.selector:
            _dict['selector'] = self.selector.to_dict()
        # override the default output from pydantic by calling `to_dict()` of template
        if self.template:
            _dict['template'] = self.template.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1JobSpec:
        """Create an instance of V1JobSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1JobSpec.parse_obj(obj)

        _obj = V1JobSpec.parse_obj({
            "active_deadline_seconds": obj.get("activeDeadlineSeconds"),
            "backoff_limit": obj.get("backoffLimit"),
            "backoff_limit_per_index": obj.get("backoffLimitPerIndex"),
            "completion_mode": obj.get("completionMode"),
            "completions": obj.get("completions"),
            "manual_selector": obj.get("manualSelector"),
            "max_failed_indexes": obj.get("maxFailedIndexes"),
            "parallelism": obj.get("parallelism"),
            "pod_failure_policy": V1PodFailurePolicy.from_dict(obj.get("podFailurePolicy")) if obj.get("podFailurePolicy") is not None else None,
            "pod_replacement_policy": obj.get("podReplacementPolicy"),
            "selector": V1LabelSelector.from_dict(obj.get("selector")) if obj.get("selector") is not None else None,
            "suspend": obj.get("suspend"),
            "template": V1PodTemplateSpec.from_dict(obj.get("template")) if obj.get("template") is not None else None,
            "ttl_seconds_after_finished": obj.get("ttlSecondsAfterFinished")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist

class V1UserInfo(BaseModel):
    """
    UserInfo holds the information about the user needed to implement the user.Info interface.  # noqa: E501
    """
    extra: Optional[Dict[str, conlist(StrictStr)]] = Field(None, description="Any additional information provided by the authenticator.")
    groups: Optional[conlist(StrictStr)] = Field(None, description="The names of groups this user is a part of.")
    uid: Optional[StrictStr] = Field(None, description="A unique value that identifies this user across time. If this user is deleted and another user by the same name is added, they will have different UIDs.")
    username: Optional[StrictStr] = Field(None, description="The name that uniquely identifies this user among all active users.")
    __properties = ["extra", "groups", "uid", "username"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1UserInfo:
        """Create an instance of V1UserInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in extra (dict of array)
        _field_dict_of_array = {}
        if self.extra:
            for _key in self.extra:
                if self.extra[_key]:
                    _field_dict_of_array[_key] = [
                        _item.to_dict() for _item in self.extra[_key]
                    ]
            _dict['extra'] = _field_dict_of_array
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1UserInfo:
        """Create an instance of V1UserInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1UserInfo.parse_obj(obj)

        _obj = V1UserInfo.parse_obj({
            "extra": obj.get("extra"),
            "groups": obj.get("groups"),
            "uid": obj.get("uid"),
            "username": obj.get("username")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v2_container_resource_metric_status import V2ContainerResourceMetricStatus
from .v2_external_metric_status import V2ExternalMetricStatus
from .v2_object_metric_status import V2ObjectMetricStatus
from .v2_pods_metric_status import V2PodsMetricStatus
from .v2_resource_metric_status import V2ResourceMetricStatus

class V2MetricStatus(BaseModel):
    """
    MetricStatus describes the last-read state of a single metric.  # noqa: E501
    """
    container_resource: Optional[V2ContainerResourceMetricStatus] = Field(None, alias="containerResource")
    external: Optional[V2ExternalMetricStatus] = None
    object: Optional[V2ObjectMetricStatus] = None
    pods: Optional[V2PodsMetricStatus] = None
    resource: Optional[V2ResourceMetricStatus] = None
    type: StrictStr = Field(..., description="type is the type of metric source.  It will be one of \"ContainerResource\", \"External\", \"Object\", \"Pods\" or \"Resource\", each corresponds to a matching field in the object. Note: \"ContainerResource\" type is available on when the feature-gate HPAContainerMetrics is enabled")
    __properties = ["containerResource", "external", "object", "pods", "resource", "type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V2MetricStatus:
        """Create an instance of V2MetricStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of container_resource
        if self.container_resource:
            _dict['containerResource'] = self.container_resource.to_dict()
        # override the default output from pydantic by calling `to_dict()` of external
        if self.external:
            _dict['external'] = self.external.to_dict()
        # override the default output from pydantic by calling `to_dict()` of object
        if self.object:
            _dict['object'] = self.object.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pods
        if self.pods:
            _dict['pods'] = self.pods.to_dict()
        # override the default output from pydantic by calling `to_dict()` of resource
        if self.resource:
            _dict['resource'] = self.resource.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V2MetricStatus:
        """Create an instance of V2MetricStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V2MetricStatus.parse_obj(obj)

        _obj = V2MetricStatus.parse_obj({
            "container_resource": V2ContainerResourceMetricStatus.from_dict(obj.get("containerResource")) if obj.get("containerResource") is not None else None,
            "external": V2ExternalMetricStatus.from_dict(obj.get("external")) if obj.get("external") is not None else None,
            "object": V2ObjectMetricStatus.from_dict(obj.get("object")) if obj.get("object") is not None else None,
            "pods": V2PodsMetricStatus.from_dict(obj.get("pods")) if obj.get("pods") is not None else None,
            "resource": V2ResourceMetricStatus.from_dict(obj.get("resource")) if obj.get("resource") is not None else None,
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr, conlist
from .v1_stateful_set_condition import V1StatefulSetCondition

class V1StatefulSetStatus(BaseModel):
    """
    StatefulSetStatus represents the current state of a StatefulSet.  # noqa: E501
    """
    available_replicas: Optional[StrictInt] = Field(None, alias="availableReplicas", description="Total number of available pods (ready for at least minReadySeconds) targeted by this statefulset.")
    collision_count: Optional[StrictInt] = Field(None, alias="collisionCount", description="collisionCount is the count of hash collisions for the StatefulSet. The StatefulSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.")
    conditions: Optional[conlist(V1StatefulSetCondition)] = Field(None, description="Represents the latest available observations of a statefulset's current state.")
    current_replicas: Optional[StrictInt] = Field(None, alias="currentReplicas", description="currentReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by currentRevision.")
    current_revision: Optional[StrictStr] = Field(None, alias="currentRevision", description="currentRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [0,currentReplicas).")
    observed_generation: Optional[StrictInt] = Field(None, alias="observedGeneration", description="observedGeneration is the most recent generation observed for this StatefulSet. It corresponds to the StatefulSet's generation, which is updated on mutation by the API Server.")
    ready_replicas: Optional[StrictInt] = Field(None, alias="readyReplicas", description="readyReplicas is the number of pods created for this StatefulSet with a Ready Condition.")
    replicas: StrictInt = Field(..., description="replicas is the number of Pods created by the StatefulSet controller.")
    update_revision: Optional[StrictStr] = Field(None, alias="updateRevision", description="updateRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [replicas-updatedReplicas,replicas)")
    updated_replicas: Optional[StrictInt] = Field(None, alias="updatedReplicas", description="updatedReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by updateRevision.")
    __properties = ["availableReplicas", "collisionCount", "conditions", "currentReplicas", "currentRevision", "observedGeneration", "readyReplicas", "replicas", "updateRevision", "updatedReplicas"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1StatefulSetStatus:
        """Create an instance of V1StatefulSetStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in conditions (list)
        _items = []
        if self.conditions:
            for _item in self.conditions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['conditions'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1StatefulSetStatus:
        """Create an instance of V1StatefulSetStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1StatefulSetStatus.parse_obj(obj)

        _obj = V1StatefulSetStatus.parse_obj({
            "available_replicas": obj.get("availableReplicas"),
            "collision_count": obj.get("collisionCount"),
            "conditions": [V1StatefulSetCondition.from_dict(_item) for _item in obj.get("conditions")] if obj.get("conditions") is not None else None,
            "current_replicas": obj.get("currentReplicas"),
            "current_revision": obj.get("currentRevision"),
            "observed_generation": obj.get("observedGeneration"),
            "ready_replicas": obj.get("readyReplicas"),
            "replicas": obj.get("replicas"),
            "update_revision": obj.get("updateRevision"),
            "updated_replicas": obj.get("updatedReplicas")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class V1GitRepoVolumeSource(BaseModel):
    """
    Represents a volume that is populated with the contents of a git repository. Git repo volumes do not support ownership management. Git repo volumes support SELinux relabeling.  DEPRECATED: GitRepo is deprecated. To provision a container with a git repo, mount an EmptyDir into an InitContainer that clones the repo using git, then mount the EmptyDir into the Pod's container.  # noqa: E501
    """
    directory: Optional[StrictStr] = Field(None, description="directory is the target directory name. Must not contain or start with '..'.  If '.' is supplied, the volume directory will be the git repository.  Otherwise, if specified, the volume will contain the git repository in the subdirectory with the given name.")
    repository: StrictStr = Field(..., description="repository is the URL")
    revision: Optional[StrictStr] = Field(None, description="revision is the commit hash for the specified revision.")
    __properties = ["directory", "repository", "revision"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1GitRepoVolumeSource:
        """Create an instance of V1GitRepoVolumeSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1GitRepoVolumeSource:
        """Create an instance of V1GitRepoVolumeSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1GitRepoVolumeSource.parse_obj(obj)

        _obj = V1GitRepoVolumeSource.parse_obj({
            "directory": obj.get("directory"),
            "repository": obj.get("repository"),
            "revision": obj.get("revision")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class V1PodResourceClaimStatus(BaseModel):
    """
    PodResourceClaimStatus is stored in the PodStatus for each PodResourceClaim which references a ResourceClaimTemplate. It stores the generated name for the corresponding ResourceClaim.  # noqa: E501
    """
    name: StrictStr = Field(..., description="Name uniquely identifies this resource claim inside the pod. This must match the name of an entry in pod.spec.resourceClaims, which implies that the string must be a DNS_LABEL.")
    resource_claim_name: Optional[StrictStr] = Field(None, alias="resourceClaimName", description="ResourceClaimName is the name of the ResourceClaim that was generated for the Pod in the namespace of the Pod. It this is unset, then generating a ResourceClaim was not necessary. The pod.spec.resourceClaims entry can be ignored in this case.")
    __properties = ["name", "resourceClaimName"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PodResourceClaimStatus:
        """Create an instance of V1PodResourceClaimStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PodResourceClaimStatus:
        """Create an instance of V1PodResourceClaimStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PodResourceClaimStatus.parse_obj(obj)

        _obj = V1PodResourceClaimStatus.parse_obj({
            "name": obj.get("name"),
            "resource_claim_name": obj.get("resourceClaimName")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr
from .v1_object_meta import V1ObjectMeta

class V1ControllerRevision(BaseModel):
    """
    ControllerRevision implements an immutable snapshot of state data. Clients are responsible for serializing and deserializing the objects that contain their internal state. Once a ControllerRevision has been successfully created, it can not be updated. The API Server will fail validation of all requests that attempt to mutate the Data field. ControllerRevisions may, however, be deleted. Note that, due to its use by both the DaemonSet and StatefulSet controllers for update and rollback, this object is beta. However, it may be subject to name and representation changes in future releases, and clients should not depend on its stability. It is primarily for internal use by controllers.  # noqa: E501
    """
    api_version: Optional[StrictStr] = Field(None, alias="apiVersion", description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources")
    data: Optional[Dict[str, Any]] = Field(None, description="Data is the serialized representation of the state.")
    kind: Optional[StrictStr] = Field(None, description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds")
    metadata: Optional[V1ObjectMeta] = None
    revision: StrictInt = Field(..., description="Revision indicates the revision of the state represented by Data.")
    __properties = ["apiVersion", "data", "kind", "metadata", "revision"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ControllerRevision:
        """Create an instance of V1ControllerRevision from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ControllerRevision:
        """Create an instance of V1ControllerRevision from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ControllerRevision.parse_obj(obj)

        _obj = V1ControllerRevision.parse_obj({
            "api_version": obj.get("apiVersion"),
            "data": obj.get("data"),
            "kind": obj.get("kind"),
            "metadata": V1ObjectMeta.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None,
            "revision": obj.get("revision")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field
from .v1_ip_block import V1IPBlock
from .v1_label_selector import V1LabelSelector

class V1NetworkPolicyPeer(BaseModel):
    """
    NetworkPolicyPeer describes a peer to allow traffic to/from. Only certain combinations of fields are allowed  # noqa: E501
    """
    ip_block: Optional[V1IPBlock] = Field(None, alias="ipBlock")
    namespace_selector: Optional[V1LabelSelector] = Field(None, alias="namespaceSelector")
    pod_selector: Optional[V1LabelSelector] = Field(None, alias="podSelector")
    __properties = ["ipBlock", "namespaceSelector", "podSelector"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1NetworkPolicyPeer:
        """Create an instance of V1NetworkPolicyPeer from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of ip_block
        if self.ip_block:
            _dict['ipBlock'] = self.ip_block.to_dict()
        # override the default output from pydantic by calling `to_dict()` of namespace_selector
        if self.namespace_selector:
            _dict['namespaceSelector'] = self.namespace_selector.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pod_selector
        if self.pod_selector:
            _dict['podSelector'] = self.pod_selector.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1NetworkPolicyPeer:
        """Create an instance of V1NetworkPolicyPeer from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1NetworkPolicyPeer.parse_obj(obj)

        _obj = V1NetworkPolicyPeer.parse_obj({
            "ip_block": V1IPBlock.from_dict(obj.get("ipBlock")) if obj.get("ipBlock") is not None else None,
            "namespace_selector": V1LabelSelector.from_dict(obj.get("namespaceSelector")) if obj.get("namespaceSelector") is not None else None,
            "pod_selector": V1LabelSelector.from_dict(obj.get("podSelector")) if obj.get("podSelector") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_csi_driver_spec import V1CSIDriverSpec
from .v1_object_meta import V1ObjectMeta

class V1CSIDriver(BaseModel):
    """
    CSIDriver captures information about a Container Storage Interface (CSI) volume driver deployed on the cluster. Kubernetes attach detach controller uses this object to determine whether attach is required. Kubelet uses this object to determine whether pod information needs to be passed on mount. CSIDriver objects are non-namespaced.  # noqa: E501
    """
    api_version: Optional[StrictStr] = Field(None, alias="apiVersion", description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources")
    kind: Optional[StrictStr] = Field(None, description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds")
    metadata: Optional[V1ObjectMeta] = None
    spec: V1CSIDriverSpec = Field(...)
    __properties = ["apiVersion", "kind", "metadata", "spec"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1CSIDriver:
        """Create an instance of V1CSIDriver from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spec
        if self.spec:
            _dict['spec'] = self.spec.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1CSIDriver:
        """Create an instance of V1CSIDriver from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1CSIDriver.parse_obj(obj)

        _obj = V1CSIDriver.parse_obj({
            "api_version": obj.get("apiVersion"),
            "kind": obj.get("kind"),
            "metadata": V1ObjectMeta.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None,
            "spec": V1CSIDriverSpec.from_dict(obj.get("spec")) if obj.get("spec") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class RbacV1Subject(BaseModel):
    """
    Subject contains a reference to the object or user identities a role binding applies to.  This can either hold a direct API object reference, or a value for non-objects such as user and group names.  # noqa: E501
    """
    api_group: Optional[StrictStr] = Field(None, alias="apiGroup", description="APIGroup holds the API group of the referenced subject. Defaults to \"\" for ServiceAccount subjects. Defaults to \"rbac.authorization.k8s.io\" for User and Group subjects.")
    kind: StrictStr = Field(..., description="Kind of object being referenced. Values defined by this API group are \"User\", \"Group\", and \"ServiceAccount\". If the Authorizer does not recognized the kind value, the Authorizer should report an error.")
    name: StrictStr = Field(..., description="Name of the object being referenced.")
    namespace: Optional[StrictStr] = Field(None, description="Namespace of the referenced object.  If the object kind is non-namespace, such as \"User\" or \"Group\", and this value is not empty the Authorizer should report an error.")
    __properties = ["apiGroup", "kind", "name", "namespace"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RbacV1Subject:
        """Create an instance of RbacV1Subject from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RbacV1Subject:
        """Create an instance of RbacV1Subject from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RbacV1Subject.parse_obj(obj)

        _obj = RbacV1Subject.parse_obj({
            "api_group": obj.get("apiGroup"),
            "kind": obj.get("kind"),
            "name": obj.get("name"),
            "namespace": obj.get("namespace")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_pod_failure_policy_on_exit_codes_requirement import V1PodFailurePolicyOnExitCodesRequirement
from .v1_pod_failure_policy_on_pod_conditions_pattern import V1PodFailurePolicyOnPodConditionsPattern

class V1PodFailurePolicyRule(BaseModel):
    """
    PodFailurePolicyRule describes how a pod failure is handled when the requirements are met. One of onExitCodes and onPodConditions, but not both, can be used in each rule.  # noqa: E501
    """
    action: StrictStr = Field(..., description="Specifies the action taken on a pod failure when the requirements are satisfied. Possible values are:  - FailJob: indicates that the pod's job is marked as Failed and all   running pods are terminated. - FailIndex: indicates that the pod's index is marked as Failed and will   not be restarted.   This value is beta-level. It can be used when the   `JobBackoffLimitPerIndex` feature gate is enabled (enabled by default). - Ignore: indicates that the counter towards the .backoffLimit is not   incremented and a replacement pod is created. - Count: indicates that the pod is handled in the default way - the   counter towards the .backoffLimit is incremented. Additional values are considered to be added in the future. Clients should react to an unknown action by skipping the rule.")
    on_exit_codes: Optional[V1PodFailurePolicyOnExitCodesRequirement] = Field(None, alias="onExitCodes")
    on_pod_conditions: Optional[conlist(V1PodFailurePolicyOnPodConditionsPattern)] = Field(None, alias="onPodConditions", description="Represents the requirement on the pod conditions. The requirement is represented as a list of pod condition patterns. The requirement is satisfied if at least one pattern matches an actual pod condition. At most 20 elements are allowed.")
    __properties = ["action", "onExitCodes", "onPodConditions"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PodFailurePolicyRule:
        """Create an instance of V1PodFailurePolicyRule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of on_exit_codes
        if self.on_exit_codes:
            _dict['onExitCodes'] = self.on_exit_codes.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in on_pod_conditions (list)
        _items = []
        if self.on_pod_conditions:
            for _item in self.on_pod_conditions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['onPodConditions'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PodFailurePolicyRule:
        """Create an instance of V1PodFailurePolicyRule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PodFailurePolicyRule.parse_obj(obj)

        _obj = V1PodFailurePolicyRule.parse_obj({
            "action": obj.get("action"),
            "on_exit_codes": V1PodFailurePolicyOnExitCodesRequirement.from_dict(obj.get("onExitCodes")) if obj.get("onExitCodes") is not None else None,
            "on_pod_conditions": [V1PodFailurePolicyOnPodConditionsPattern.from_dict(_item) for _item in obj.get("onPodConditions")] if obj.get("onPodConditions") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr

class V1ContainerPort(BaseModel):
    """
    ContainerPort represents a network port in a single container.  # noqa: E501
    """
    container_port: StrictInt = Field(..., alias="containerPort", description="Number of port to expose on the pod's IP address. This must be a valid port number, 0 < x < 65536.")
    host_ip: Optional[StrictStr] = Field(None, alias="hostIP", description="What host IP to bind the external port to.")
    host_port: Optional[StrictInt] = Field(None, alias="hostPort", description="Number of port to expose on the host. If specified, this must be a valid port number, 0 < x < 65536. If HostNetwork is specified, this must match ContainerPort. Most containers do not need this.")
    name: Optional[StrictStr] = Field(None, description="If specified, this must be an IANA_SVC_NAME and unique within the pod. Each named port in a pod must have a unique name. Name for the port that can be referred to by services.")
    protocol: Optional[StrictStr] = Field(None, description="Protocol for port. Must be UDP, TCP, or SCTP. Defaults to \"TCP\".")
    __properties = ["containerPort", "hostIP", "hostPort", "name", "protocol"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ContainerPort:
        """Create an instance of V1ContainerPort from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ContainerPort:
        """Create an instance of V1ContainerPort from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ContainerPort.parse_obj(obj)

        _obj = V1ContainerPort.parse_obj({
            "container_port": obj.get("containerPort"),
            "host_ip": obj.get("hostIP"),
            "host_port": obj.get("hostPort"),
            "name": obj.get("name"),
            "protocol": obj.get("protocol")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class V1SeccompProfile(BaseModel):
    """
    SeccompProfile defines a pod/container's seccomp profile settings. Only one profile source may be set.  # noqa: E501
    """
    localhost_profile: Optional[StrictStr] = Field(None, alias="localhostProfile", description="localhostProfile indicates a profile defined in a file on the node should be used. The profile must be preconfigured on the node to work. Must be a descending path, relative to the kubelet's configured seccomp profile location. Must be set if type is \"Localhost\". Must NOT be set for any other type.")
    type: StrictStr = Field(..., description="type indicates which kind of seccomp profile will be applied. Valid options are:  Localhost - a profile defined in a file on the node should be used. RuntimeDefault - the container runtime default profile should be used. Unconfined - no profile should be applied.")
    __properties = ["localhostProfile", "type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1SeccompProfile:
        """Create an instance of V1SeccompProfile from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1SeccompProfile:
        """Create an instance of V1SeccompProfile from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1SeccompProfile.parse_obj(obj)

        _obj = V1SeccompProfile.parse_obj({
            "localhost_profile": obj.get("localhostProfile"),
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr
from .v1_object_meta import V1ObjectMeta
from .v1alpha1_ip_address_spec import V1alpha1IPAddressSpec

class V1alpha1IPAddress(BaseModel):
    """
    IPAddress represents a single IP of a single IP Family. The object is designed to be used by APIs that operate on IP addresses. The object is used by the Service core API for allocation of IP addresses. An IP address can be represented in different formats, to guarantee the uniqueness of the IP, the name of the object is the IP address in canonical format, four decimal digits separated by dots suppressing leading zeros for IPv4 and the representation defined by RFC 5952 for IPv6. Valid: 192.168.1.5 or 2001:db8::1 or 2001:db8:aaaa:bbbb:cccc:dddd:eeee:1 Invalid: 10.01.2.3 or 2001:db8:0:0:0::1  # noqa: E501
    """
    api_version: Optional[StrictStr] = Field(None, alias="apiVersion", description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources")
    kind: Optional[StrictStr] = Field(None, description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds")
    metadata: Optional[V1ObjectMeta] = None
    spec: Optional[V1alpha1IPAddressSpec] = None
    __properties = ["apiVersion", "kind", "metadata", "spec"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1alpha1IPAddress:
        """Create an instance of V1alpha1IPAddress from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of spec
        if self.spec:
            _dict['spec'] = self.spec.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1alpha1IPAddress:
        """Create an instance of V1alpha1IPAddress from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1alpha1IPAddress.parse_obj(obj)

        _obj = V1alpha1IPAddress.parse_obj({
            "api_version": obj.get("apiVersion"),
            "kind": obj.get("kind"),
            "metadata": V1ObjectMeta.from_dict(obj.get("metadata")) if obj.get("metadata") is not None else None,
            "spec": V1alpha1IPAddressSpec.from_dict(obj.get("spec")) if obj.get("spec") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist
from .v1alpha2_allocation_result import V1alpha2AllocationResult
from .v1alpha2_resource_claim_consumer_reference import V1alpha2ResourceClaimConsumerReference

class V1alpha2ResourceClaimStatus(BaseModel):
    """
    ResourceClaimStatus tracks whether the resource has been allocated and what the resulting attributes are.  # noqa: E501
    """
    allocation: Optional[V1alpha2AllocationResult] = None
    deallocation_requested: Optional[StrictBool] = Field(None, alias="deallocationRequested", description="DeallocationRequested indicates that a ResourceClaim is to be deallocated.  The driver then must deallocate this claim and reset the field together with clearing the Allocation field.  While DeallocationRequested is set, no new consumers may be added to ReservedFor.")
    driver_name: Optional[StrictStr] = Field(None, alias="driverName", description="DriverName is a copy of the driver name from the ResourceClass at the time when allocation started.")
    reserved_for: Optional[conlist(V1alpha2ResourceClaimConsumerReference)] = Field(None, alias="reservedFor", description="ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started.  There can be at most 32 such reservations. This may get increased in the future, but not reduced.")
    __properties = ["allocation", "deallocationRequested", "driverName", "reservedFor"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1alpha2ResourceClaimStatus:
        """Create an instance of V1alpha2ResourceClaimStatus from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of allocation
        if self.allocation:
            _dict['allocation'] = self.allocation.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in reserved_for (list)
        _items = []
        if self.reserved_for:
            for _item in self.reserved_for:
                if _item:
                    _items.append(_item.to_dict())
            _dict['reservedFor'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1alpha2ResourceClaimStatus:
        """Create an instance of V1alpha2ResourceClaimStatus from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1alpha2ResourceClaimStatus.parse_obj(obj)

        _obj = V1alpha2ResourceClaimStatus.parse_obj({
            "allocation": V1alpha2AllocationResult.from_dict(obj.get("allocation")) if obj.get("allocation") is not None else None,
            "deallocation_requested": obj.get("deallocationRequested"),
            "driver_name": obj.get("driverName"),
            "reserved_for": [V1alpha2ResourceClaimConsumerReference.from_dict(_item) for _item in obj.get("reservedFor")] if obj.get("reservedFor") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr
from .v1_object_field_selector import V1ObjectFieldSelector
from .v1_resource_field_selector import V1ResourceFieldSelector

class V1DownwardAPIVolumeFile(BaseModel):
    """
    DownwardAPIVolumeFile represents information to create the file containing the pod field  # noqa: E501
    """
    field_ref: Optional[V1ObjectFieldSelector] = Field(None, alias="fieldRef")
    mode: Optional[StrictInt] = Field(None, description="Optional: mode bits used to set permissions on this file, must be an octal value between 0000 and 0777 or a decimal value between 0 and 511. YAML accepts both octal and decimal values, JSON requires decimal values for mode bits. If not specified, the volume defaultMode will be used. This might be in conflict with other options that affect the file mode, like fsGroup, and the result can be other mode bits set.")
    path: StrictStr = Field(..., description="Required: Path is  the relative path name of the file to be created. Must not be absolute or contain the '..' path. Must be utf-8 encoded. The first item of the relative path must not start with '..'")
    resource_field_ref: Optional[V1ResourceFieldSelector] = Field(None, alias="resourceFieldRef")
    __properties = ["fieldRef", "mode", "path", "resourceFieldRef"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1DownwardAPIVolumeFile:
        """Create an instance of V1DownwardAPIVolumeFile from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of field_ref
        if self.field_ref:
            _dict['fieldRef'] = self.field_ref.to_dict()
        # override the default output from pydantic by calling `to_dict()` of resource_field_ref
        if self.resource_field_ref:
            _dict['resourceFieldRef'] = self.resource_field_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1DownwardAPIVolumeFile:
        """Create an instance of V1DownwardAPIVolumeFile from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1DownwardAPIVolumeFile.parse_obj(obj)

        _obj = V1DownwardAPIVolumeFile.parse_obj({
            "field_ref": V1ObjectFieldSelector.from_dict(obj.get("fieldRef")) if obj.get("fieldRef") is not None else None,
            "mode": obj.get("mode"),
            "path": obj.get("path"),
            "resource_field_ref": V1ResourceFieldSelector.from_dict(obj.get("resourceFieldRef")) if obj.get("resourceFieldRef") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictInt
from .v1_cross_version_object_reference import V1CrossVersionObjectReference

class V1HorizontalPodAutoscalerSpec(BaseModel):
    """
    specification of a horizontal pod autoscaler.  # noqa: E501
    """
    max_replicas: StrictInt = Field(..., alias="maxReplicas", description="maxReplicas is the upper limit for the number of pods that can be set by the autoscaler; cannot be smaller than MinReplicas.")
    min_replicas: Optional[StrictInt] = Field(None, alias="minReplicas", description="minReplicas is the lower limit for the number of replicas to which the autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one Object or External metric is configured.  Scaling is active as long as at least one metric value is available.")
    scale_target_ref: V1CrossVersionObjectReference = Field(..., alias="scaleTargetRef")
    target_cpu_utilization_percentage: Optional[StrictInt] = Field(None, alias="targetCPUUtilizationPercentage", description="targetCPUUtilizationPercentage is the target average CPU utilization (represented as a percentage of requested CPU) over all the pods; if not specified the default autoscaling policy will be used.")
    __properties = ["maxReplicas", "minReplicas", "scaleTargetRef", "targetCPUUtilizationPercentage"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1HorizontalPodAutoscalerSpec:
        """Create an instance of V1HorizontalPodAutoscalerSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of scale_target_ref
        if self.scale_target_ref:
            _dict['scaleTargetRef'] = self.scale_target_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1HorizontalPodAutoscalerSpec:
        """Create an instance of V1HorizontalPodAutoscalerSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1HorizontalPodAutoscalerSpec.parse_obj(obj)

        _obj = V1HorizontalPodAutoscalerSpec.parse_obj({
            "max_replicas": obj.get("maxReplicas"),
            "min_replicas": obj.get("minReplicas"),
            "scale_target_ref": V1CrossVersionObjectReference.from_dict(obj.get("scaleTargetRef")) if obj.get("scaleTargetRef") is not None else None,
            "target_cpu_utilization_percentage": obj.get("targetCPUUtilizationPercentage")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_http_header import V1HTTPHeader

class V1HTTPGetAction(BaseModel):
    """
    HTTPGetAction describes an action based on HTTP Get requests.  # noqa: E501
    """
    host: Optional[StrictStr] = Field(None, description="Host name to connect to, defaults to the pod IP. You probably want to set \"Host\" in httpHeaders instead.")
    http_headers: Optional[conlist(V1HTTPHeader)] = Field(None, alias="httpHeaders", description="Custom headers to set in the request. HTTP allows repeated headers.")
    path: Optional[StrictStr] = Field(None, description="Path to access on the HTTP server.")
    port: Dict[str, Any] = Field(..., description="Name or number of the port to access on the container. Number must be in the range 1 to 65535. Name must be an IANA_SVC_NAME.")
    scheme: Optional[StrictStr] = Field(None, description="Scheme to use for connecting to the host. Defaults to HTTP.")
    __properties = ["host", "httpHeaders", "path", "port", "scheme"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1HTTPGetAction:
        """Create an instance of V1HTTPGetAction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in http_headers (list)
        _items = []
        if self.http_headers:
            for _item in self.http_headers:
                if _item:
                    _items.append(_item.to_dict())
            _dict['httpHeaders'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1HTTPGetAction:
        """Create an instance of V1HTTPGetAction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1HTTPGetAction.parse_obj(obj)

        _obj = V1HTTPGetAction.parse_obj({
            "host": obj.get("host"),
            "http_headers": [V1HTTPHeader.from_dict(_item) for _item in obj.get("httpHeaders")] if obj.get("httpHeaders") is not None else None,
            "path": obj.get("path"),
            "port": obj.get("port"),
            "scheme": obj.get("scheme")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr
from .v1_capabilities import V1Capabilities
from .v1_se_linux_options import V1SELinuxOptions
from .v1_seccomp_profile import V1SeccompProfile
from .v1_windows_security_context_options import V1WindowsSecurityContextOptions

class V1SecurityContext(BaseModel):
    """
    SecurityContext holds security configuration that will be applied to a container. Some fields are present in both SecurityContext and PodSecurityContext.  When both are set, the values in SecurityContext take precedence.  # noqa: E501
    """
    allow_privilege_escalation: Optional[StrictBool] = Field(None, alias="allowPrivilegeEscalation", description="AllowPrivilegeEscalation controls whether a process can gain more privileges than its parent process. This bool directly controls if the no_new_privs flag will be set on the container process. AllowPrivilegeEscalation is true always when the container is: 1) run as Privileged 2) has CAP_SYS_ADMIN Note that this field cannot be set when spec.os.name is windows.")
    capabilities: Optional[V1Capabilities] = None
    privileged: Optional[StrictBool] = Field(None, description="Run container in privileged mode. Processes in privileged containers are essentially equivalent to root on the host. Defaults to false. Note that this field cannot be set when spec.os.name is windows.")
    proc_mount: Optional[StrictStr] = Field(None, alias="procMount", description="procMount denotes the type of proc mount to use for the containers. The default is DefaultProcMount which uses the container runtime defaults for readonly paths and masked paths. This requires the ProcMountType feature flag to be enabled. Note that this field cannot be set when spec.os.name is windows.")
    read_only_root_filesystem: Optional[StrictBool] = Field(None, alias="readOnlyRootFilesystem", description="Whether this container has a read-only root filesystem. Default is false. Note that this field cannot be set when spec.os.name is windows.")
    run_as_group: Optional[StrictInt] = Field(None, alias="runAsGroup", description="The GID to run the entrypoint of the container process. Uses runtime default if unset. May also be set in PodSecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence. Note that this field cannot be set when spec.os.name is windows.")
    run_as_non_root: Optional[StrictBool] = Field(None, alias="runAsNonRoot", description="Indicates that the container must run as a non-root user. If true, the Kubelet will validate the image at runtime to ensure that it does not run as UID 0 (root) and fail to start the container if it does. If unset or false, no such validation will be performed. May also be set in PodSecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.")
    run_as_user: Optional[StrictInt] = Field(None, alias="runAsUser", description="The UID to run the entrypoint of the container process. Defaults to user specified in image metadata if unspecified. May also be set in PodSecurityContext.  If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence. Note that this field cannot be set when spec.os.name is windows.")
    se_linux_options: Optional[V1SELinuxOptions] = Field(None, alias="seLinuxOptions")
    seccomp_profile: Optional[V1SeccompProfile] = Field(None, alias="seccompProfile")
    windows_options: Optional[V1WindowsSecurityContextOptions] = Field(None, alias="windowsOptions")
    __properties = ["allowPrivilegeEscalation", "capabilities", "privileged", "procMount", "readOnlyRootFilesystem", "runAsGroup", "runAsNonRoot", "runAsUser", "seLinuxOptions", "seccompProfile", "windowsOptions"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1SecurityContext:
        """Create an instance of V1SecurityContext from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of capabilities
        if self.capabilities:
            _dict['capabilities'] = self.capabilities.to_dict()
        # override the default output from pydantic by calling `to_dict()` of se_linux_options
        if self.se_linux_options:
            _dict['seLinuxOptions'] = self.se_linux_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of seccomp_profile
        if self.seccomp_profile:
            _dict['seccompProfile'] = self.seccomp_profile.to_dict()
        # override the default output from pydantic by calling `to_dict()` of windows_options
        if self.windows_options:
            _dict['windowsOptions'] = self.windows_options.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1SecurityContext:
        """Create an instance of V1SecurityContext from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1SecurityContext.parse_obj(obj)

        _obj = V1SecurityContext.parse_obj({
            "allow_privilege_escalation": obj.get("allowPrivilegeEscalation"),
            "capabilities": V1Capabilities.from_dict(obj.get("capabilities")) if obj.get("capabilities") is not None else None,
            "privileged": obj.get("privileged"),
            "proc_mount": obj.get("procMount"),
            "read_only_root_filesystem": obj.get("readOnlyRootFilesystem"),
            "run_as_group": obj.get("runAsGroup"),
            "run_as_non_root": obj.get("runAsNonRoot"),
            "run_as_user": obj.get("runAsUser"),
            "se_linux_options": V1SELinuxOptions.from_dict(obj.get("seLinuxOptions")) if obj.get("seLinuxOptions") is not None else None,
            "seccomp_profile": V1SeccompProfile.from_dict(obj.get("seccompProfile")) if obj.get("seccompProfile") is not None else None,
            "windows_options": V1WindowsSecurityContextOptions.from_dict(obj.get("windowsOptions")) if obj.get("windowsOptions") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_pod_dns_config_option import V1PodDNSConfigOption

class V1PodDNSConfig(BaseModel):
    """
    PodDNSConfig defines the DNS parameters of a pod in addition to those generated from DNSPolicy.  # noqa: E501
    """
    nameservers: Optional[conlist(StrictStr)] = Field(None, description="A list of DNS name server IP addresses. This will be appended to the base nameservers generated from DNSPolicy. Duplicated nameservers will be removed.")
    options: Optional[conlist(V1PodDNSConfigOption)] = Field(None, description="A list of DNS resolver options. This will be merged with the base options generated from DNSPolicy. Duplicated entries will be removed. Resolution options given in Options will override those that appear in the base DNSPolicy.")
    searches: Optional[conlist(StrictStr)] = Field(None, description="A list of DNS search domains for host-name lookup. This will be appended to the base search paths generated from DNSPolicy. Duplicated search paths will be removed.")
    __properties = ["nameservers", "options", "searches"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1PodDNSConfig:
        """Create an instance of V1PodDNSConfig from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in options (list)
        _items = []
        if self.options:
            for _item in self.options:
                if _item:
                    _items.append(_item.to_dict())
            _dict['options'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1PodDNSConfig:
        """Create an instance of V1PodDNSConfig from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1PodDNSConfig.parse_obj(obj)

        _obj = V1PodDNSConfig.parse_obj({
            "nameservers": obj.get("nameservers"),
            "options": [V1PodDNSConfigOption.from_dict(_item) for _item in obj.get("options")] if obj.get("options") is not None else None,
            "searches": obj.get("searches")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr
from .v1_local_object_reference import V1LocalObjectReference

class V1FlexVolumeSource(BaseModel):
    """
    FlexVolume represents a generic volume resource that is provisioned/attached using an exec based plugin.  # noqa: E501
    """
    driver: StrictStr = Field(..., description="driver is the name of the driver to use for this volume.")
    fs_type: Optional[StrictStr] = Field(None, alias="fsType", description="fsType is the filesystem type to mount. Must be a filesystem type supported by the host operating system. Ex. \"ext4\", \"xfs\", \"ntfs\". The default filesystem depends on FlexVolume script.")
    options: Optional[Dict[str, StrictStr]] = Field(None, description="options is Optional: this field holds extra command options if any.")
    read_only: Optional[StrictBool] = Field(None, alias="readOnly", description="readOnly is Optional: defaults to false (read/write). ReadOnly here will force the ReadOnly setting in VolumeMounts.")
    secret_ref: Optional[V1LocalObjectReference] = Field(None, alias="secretRef")
    __properties = ["driver", "fsType", "options", "readOnly", "secretRef"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1FlexVolumeSource:
        """Create an instance of V1FlexVolumeSource from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of secret_ref
        if self.secret_ref:
            _dict['secretRef'] = self.secret_ref.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1FlexVolumeSource:
        """Create an instance of V1FlexVolumeSource from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1FlexVolumeSource.parse_obj(obj)

        _obj = V1FlexVolumeSource.parse_obj({
            "driver": obj.get("driver"),
            "fs_type": obj.get("fsType"),
            "options": obj.get("options"),
            "read_only": obj.get("readOnly"),
            "secret_ref": V1LocalObjectReference.from_dict(obj.get("secretRef")) if obj.get("secretRef") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from .v1_label_selector import V1LabelSelector
from .v1_network_policy_egress_rule import V1NetworkPolicyEgressRule
from .v1_network_policy_ingress_rule import V1NetworkPolicyIngressRule

class V1NetworkPolicySpec(BaseModel):
    """
    NetworkPolicySpec provides the specification of a NetworkPolicy  # noqa: E501
    """
    egress: Optional[conlist(V1NetworkPolicyEgressRule)] = Field(None, description="egress is a list of egress rules to be applied to the selected pods. Outgoing traffic is allowed if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic matches at least one egress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy limits all outgoing traffic (and serves solely to ensure that the pods it selects are isolated by default). This field is beta-level in 1.8")
    ingress: Optional[conlist(V1NetworkPolicyIngressRule)] = Field(None, description="ingress is a list of ingress rules to be applied to the selected pods. Traffic is allowed to a pod if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic source is the pod's local node, OR if the traffic matches at least one ingress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy does not allow any traffic (and serves solely to ensure that the pods it selects are isolated by default)")
    pod_selector: V1LabelSelector = Field(..., alias="podSelector")
    policy_types: Optional[conlist(StrictStr)] = Field(None, alias="policyTypes", description="policyTypes is a list of rule types that the NetworkPolicy relates to. Valid options are [\"Ingress\"], [\"Egress\"], or [\"Ingress\", \"Egress\"]. If this field is not specified, it will default based on the existence of ingress or egress rules; policies that contain an egress section are assumed to affect egress, and all policies (whether or not they contain an ingress section) are assumed to affect ingress. If you want to write an egress-only policy, you must explicitly specify policyTypes [ \"Egress\" ]. Likewise, if you want to write a policy that specifies that no egress is allowed, you must specify a policyTypes value that include \"Egress\" (since such a policy would not include an egress section and would otherwise default to just [ \"Ingress\" ]). This field is beta-level in 1.8")
    __properties = ["egress", "ingress", "podSelector", "policyTypes"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1NetworkPolicySpec:
        """Create an instance of V1NetworkPolicySpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in egress (list)
        _items = []
        if self.egress:
            for _item in self.egress:
                if _item:
                    _items.append(_item.to_dict())
            _dict['egress'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in ingress (list)
        _items = []
        if self.ingress:
            for _item in self.ingress:
                if _item:
                    _items.append(_item.to_dict())
            _dict['ingress'] = _items
        # override the default output from pydantic by calling `to_dict()` of pod_selector
        if self.pod_selector:
            _dict['podSelector'] = self.pod_selector.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1NetworkPolicySpec:
        """Create an instance of V1NetworkPolicySpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1NetworkPolicySpec.parse_obj(obj)

        _obj = V1NetworkPolicySpec.parse_obj({
            "egress": [V1NetworkPolicyEgressRule.from_dict(_item) for _item in obj.get("egress")] if obj.get("egress") is not None else None,
            "ingress": [V1NetworkPolicyIngressRule.from_dict(_item) for _item in obj.get("ingress")] if obj.get("ingress") is not None else None,
            "pod_selector": V1LabelSelector.from_dict(obj.get("podSelector")) if obj.get("podSelector") is not None else None,
            "policy_types": obj.get("policyTypes")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist

class V1RuleWithOperations(BaseModel):
    """
    RuleWithOperations is a tuple of Operations and Resources. It is recommended to make sure that all the tuple expansions are valid.  # noqa: E501
    """
    api_groups: Optional[conlist(StrictStr)] = Field(None, alias="apiGroups", description="APIGroups is the API groups the resources belong to. '*' is all groups. If '*' is present, the length of the slice must be one. Required.")
    api_versions: Optional[conlist(StrictStr)] = Field(None, alias="apiVersions", description="APIVersions is the API versions the resources belong to. '*' is all versions. If '*' is present, the length of the slice must be one. Required.")
    operations: Optional[conlist(StrictStr)] = Field(None, description="Operations is the operations the admission hook cares about - CREATE, UPDATE, DELETE, CONNECT or * for all of those operations and any future admission operations that are added. If '*' is present, the length of the slice must be one. Required.")
    resources: Optional[conlist(StrictStr)] = Field(None, description="Resources is a list of resources this rule applies to.  For example: 'pods' means pods. 'pods/log' means the log subresource of pods. '*' means all resources, but not subresources. 'pods/*' means all subresources of pods. '*/scale' means all scale subresources. '*/*' means all resources and their subresources.  If wildcard is present, the validation rule will ensure resources do not overlap with each other.  Depending on the enclosing object, subresources might not be allowed. Required.")
    scope: Optional[StrictStr] = Field(None, description="scope specifies the scope of this rule. Valid values are \"Cluster\", \"Namespaced\", and \"*\" \"Cluster\" means that only cluster-scoped resources will match this rule. Namespace API objects are cluster-scoped. \"Namespaced\" means that only namespaced resources will match this rule. \"*\" means that there are no scope restrictions. Subresources match the scope of their parent resource. Default is \"*\".")
    __properties = ["apiGroups", "apiVersions", "operations", "resources", "scope"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1RuleWithOperations:
        """Create an instance of V1RuleWithOperations from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1RuleWithOperations:
        """Create an instance of V1RuleWithOperations from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1RuleWithOperations.parse_obj(obj)

        _obj = V1RuleWithOperations.parse_obj({
            "api_groups": obj.get("apiGroups"),
            "api_versions": obj.get("apiVersions"),
            "operations": obj.get("operations"),
            "resources": obj.get("resources"),
            "scope": obj.get("scope")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class V1beta3PriorityLevelConfigurationCondition(BaseModel):
    """
    PriorityLevelConfigurationCondition defines the condition of priority level.  # noqa: E501
    """
    last_transition_time: Optional[datetime] = Field(None, alias="lastTransitionTime", description="`lastTransitionTime` is the last time the condition transitioned from one status to another.")
    message: Optional[StrictStr] = Field(None, description="`message` is a human-readable message indicating details about last transition.")
    reason: Optional[StrictStr] = Field(None, description="`reason` is a unique, one-word, CamelCase reason for the condition's last transition.")
    status: Optional[StrictStr] = Field(None, description="`status` is the status of the condition. Can be True, False, Unknown. Required.")
    type: Optional[StrictStr] = Field(None, description="`type` is the type of the condition. Required.")
    __properties = ["lastTransitionTime", "message", "reason", "status", "type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1beta3PriorityLevelConfigurationCondition:
        """Create an instance of V1beta3PriorityLevelConfigurationCondition from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1beta3PriorityLevelConfigurationCondition:
        """Create an instance of V1beta3PriorityLevelConfigurationCondition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1beta3PriorityLevelConfigurationCondition.parse_obj(obj)

        _obj = V1beta3PriorityLevelConfigurationCondition.parse_obj({
            "last_transition_time": obj.get("lastTransitionTime"),
            "message": obj.get("message"),
            "reason": obj.get("reason"),
            "status": obj.get("status"),
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field
from .v1_exec_action import V1ExecAction
from .v1_http_get_action import V1HTTPGetAction
from .v1_sleep_action import V1SleepAction
from .v1_tcp_socket_action import V1TCPSocketAction

class V1LifecycleHandler(BaseModel):
    """
    LifecycleHandler defines a specific action that should be taken in a lifecycle hook. One and only one of the fields, except TCPSocket must be specified.  # noqa: E501
    """
    var_exec: Optional[V1ExecAction] = Field(None, alias="exec")
    http_get: Optional[V1HTTPGetAction] = Field(None, alias="httpGet")
    sleep: Optional[V1SleepAction] = None
    tcp_socket: Optional[V1TCPSocketAction] = Field(None, alias="tcpSocket")
    __properties = ["exec", "httpGet", "sleep", "tcpSocket"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1LifecycleHandler:
        """Create an instance of V1LifecycleHandler from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of var_exec
        if self.var_exec:
            _dict['exec'] = self.var_exec.to_dict()
        # override the default output from pydantic by calling `to_dict()` of http_get
        if self.http_get:
            _dict['httpGet'] = self.http_get.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sleep
        if self.sleep:
            _dict['sleep'] = self.sleep.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tcp_socket
        if self.tcp_socket:
            _dict['tcpSocket'] = self.tcp_socket.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1LifecycleHandler:
        """Create an instance of V1LifecycleHandler from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1LifecycleHandler.parse_obj(obj)

        _obj = V1LifecycleHandler.parse_obj({
            "var_exec": V1ExecAction.from_dict(obj.get("exec")) if obj.get("exec") is not None else None,
            "http_get": V1HTTPGetAction.from_dict(obj.get("httpGet")) if obj.get("httpGet") is not None else None,
            "sleep": V1SleepAction.from_dict(obj.get("sleep")) if obj.get("sleep") is not None else None,
            "tcp_socket": V1TCPSocketAction.from_dict(obj.get("tcpSocket")) if obj.get("tcpSocket") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist
from .v1_node_config_source import V1NodeConfigSource
from .v1_taint import V1Taint

class V1NodeSpec(BaseModel):
    """
    NodeSpec describes the attributes that a node is created with.  # noqa: E501
    """
    config_source: Optional[V1NodeConfigSource] = Field(None, alias="configSource")
    external_id: Optional[StrictStr] = Field(None, alias="externalID", description="Deprecated. Not all kubelets will set this field. Remove field after 1.13. see: https://issues.k8s.io/61966")
    pod_cidr: Optional[StrictStr] = Field(None, alias="podCIDR", description="PodCIDR represents the pod IP range assigned to the node.")
    pod_cidrs: Optional[conlist(StrictStr)] = Field(None, alias="podCIDRs", description="podCIDRs represents the IP ranges assigned to the node for usage by Pods on that node. If this field is specified, the 0th entry must match the podCIDR field. It may contain at most 1 value for each of IPv4 and IPv6.")
    provider_id: Optional[StrictStr] = Field(None, alias="providerID", description="ID of the node assigned by the cloud provider in the format: <ProviderName>://<ProviderSpecificNodeID>")
    taints: Optional[conlist(V1Taint)] = Field(None, description="If specified, the node's taints.")
    unschedulable: Optional[StrictBool] = Field(None, description="Unschedulable controls node schedulability of new pods. By default, node is schedulable. More info: https://kubernetes.io/docs/concepts/nodes/node/#manual-node-administration")
    __properties = ["configSource", "externalID", "podCIDR", "podCIDRs", "providerID", "taints", "unschedulable"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1NodeSpec:
        """Create an instance of V1NodeSpec from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of config_source
        if self.config_source:
            _dict['configSource'] = self.config_source.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in taints (list)
        _items = []
        if self.taints:
            for _item in self.taints:
                if _item:
                    _items.append(_item.to_dict())
            _dict['taints'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1NodeSpec:
        """Create an instance of V1NodeSpec from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1NodeSpec.parse_obj(obj)

        _obj = V1NodeSpec.parse_obj({
            "config_source": V1NodeConfigSource.from_dict(obj.get("configSource")) if obj.get("configSource") is not None else None,
            "external_id": obj.get("externalID"),
            "pod_cidr": obj.get("podCIDR"),
            "pod_cidrs": obj.get("podCIDRs"),
            "provider_id": obj.get("providerID"),
            "taints": [V1Taint.from_dict(_item) for _item in obj.get("taints")] if obj.get("taints") is not None else None,
            "unschedulable": obj.get("unschedulable")
        })
        return _obj





# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: release-1.29
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist

class V1ResourceRule(BaseModel):
    """
    ResourceRule is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.  # noqa: E501
    """
    api_groups: Optional[conlist(StrictStr)] = Field(None, alias="apiGroups", description="APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.  \"*\" means all.")
    resource_names: Optional[conlist(StrictStr)] = Field(None, alias="resourceNames", description="ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.  \"*\" means all.")
    resources: Optional[conlist(StrictStr)] = Field(None, description="Resources is a list of resources this rule applies to.  \"*\" means all in the specified apiGroups.  \"*/foo\" represents the subresource 'foo' for all resources in the specified apiGroups.")
    verbs: conlist(StrictStr) = Field(..., description="Verb is a list of kubernetes resource API verbs, like: get, list, watch, create, update, delete, proxy.  \"*\" means all.")
    __properties = ["apiGroups", "resourceNames", "resources", "verbs"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> V1ResourceRule:
        """Create an instance of V1ResourceRule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> V1ResourceRule:
        """Create an instance of V1ResourceRule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return V1ResourceRule.parse_obj(obj)

        _obj = V1ResourceRule.parse_obj({
            "api_groups": obj.get("apiGroups"),
            "resource_names": obj.get("resourceNames"),
            "resources": obj.get("resources"),
            "verbs": obj.get("verbs")
        })
        return _obj


